function paper_RBFGS_RWRBFGS()
% Generate the table in the RBroydenBasic paper.
% Use Brockett problem to compare the our RBFGS to the Ring and Wirth's
% RBFGS
%
% By Wen Huang
    tab = zeros(9, 8);
    methods = [2, 10, 2, 10, 2, 10, 2, 10];
    ns = [12, 12, 12, 12, 24, 24, 24, 24];
    ps = [6, 6, 12, 12, 12, 12, 24, 24];
    randseedsset = [1,3,5,7,9,11,13,15,17,19];
    row = 0;
    for i = 1 : 8
        tab(row + 1, i) = 0;
        tab(row + 2, i) = 0;
        tab(row + 3, i) = 0;
        tab(row + 4, i) = 0;
        tab(row + 5, i) = 0;
        tab(row + 6, i) = 0;
        tab(row + 7, i) = 0;
        tab(row + 8, i) = 0;
        tab(row + 9, i) = 0;
        for j = 1 : length(randseedsset)
            [X, F, G, T, timecost, nf, ng, nR, nH, nV, nVp, ave_time] = test_Brockett(methods(i), ns(i), ps(i), randseedsset(j));
            tab(row + 1, i) = tab(row + 1, i)+ length(X);
            tab(row + 2, i) = tab(row + 2, i) + nf;
            tab(row + 3, i) = tab(row + 3, i) + ng;
            tab(row + 4, i) = tab(row + 4, i) + nH;
            tab(row + 5, i) = tab(row + 5, i) + nV + nVp;
            tab(row + 6, i) = tab(row + 6, i) + nR;
            tab(row + 7, i) = tab(row + 7, i) + G(end);
            tab(row + 8, i) = tab(row + 8, i) + G(end)/G(1);
            tab(row + 9, i) = tab(row + 9, i) + ave_time;
        end
        tab(row + 1, i) = tab(row + 1, i) / length(randseedsset);
        tab(row + 2, i) = tab(row + 2, i) / length(randseedsset);
        tab(row + 3, i) = tab(row + 3, i) / length(randseedsset);
        tab(row + 4, i) = tab(row + 4, i) / length(randseedsset);
        tab(row + 5, i) = tab(row + 5, i) / length(randseedsset);
        tab(row + 6, i) = tab(row + 6, i) / length(randseedsset);
        tab(row + 7, i) = tab(row + 7, i) / length(randseedsset);
        tab(row + 8, i) = tab(row + 8, i) / length(randseedsset);
        tab(row + 9, i) = tab(row + 9, i) / length(randseedsset);
%         stiefelGS{i} = G;
    end
    
    fout = fopen('paper_RBFGS_RWRBFGS_tab.txt','w');
    for i = 1 : 9
        for j = 1 : 8
            if(mod(i-1, 9) < 0)
                if(j < 8)
                    fprintf(fout,'%d & ',tab(i, j));
                else
                    fprintf(fout,'%d',tab(i, j));
                end
            else
                tabstr = outputfloat(tab(i, j));
                if(j < 8)
                    fprintf(fout,['$' tabstr '$ & ']);
                else
                    fprintf(fout,['$' tabstr '$']);
                end
            end
        end
        fprintf(fout,' \\\\\n');
    end
    fclose(fout);
end

function str = outputfloat(x)
    if(x <= 0)
        str = '';
        return;
    end
    p = log(x)/log(10);
    p = - ceil(-p);
    x = round(x * 10^(-p) * 100);
    x = x / 100;
    strx = sprintf('%3.2f', x);
    if(p ~= 0)
        str = [strx '_{' num2str(p) '}'];
    else
        str = strx;
    end
end

function [X, F, G, T, timecost, nf, ng, nR, nH, nV, nVp, ave_time] = test_Brockett(method, n, p, randseed)
    fprintf('test for Brockett problem. \n');
    fprintf('X^* = arg min_{x in St(p, n)} trace(X^T B X N), where B is an n by n matrix. \n');
    r = rand();
    r = floor(r * 10000);
%     r = 1;
    r = randseed;
    rand('state', r);
    randn('state', r);
    fprintf('In this case, n = %d, p = %d. B and x0 are chosen randomly. Random seed is %d \n', n, p, r);
    B = randn(n, n);
    B = B + B';
    N = diag(1:p);
    x1U = randn(n, p);
    [x1.U, r] = qr(x1U, 0);
    if(method == 2)
        Retraction = 6;
        VT = 7;
    else
        Retraction = 4;
        VT = 7;
    end
    if(VT == 7 || VT == 13 || VT == 19 || VT == 25)
        d = 0.5 * p * (p - 1) + (n - p) * p;
        H0 = eye(d);
        x1.Up = null(x1.U');
    end
    params.x0 = x1;
    params.H0 = H0;
    params.error = 1e-6;
    params.StopCriterion = 3;
    params.max_t = 20000;
    params.debug = 0;
    params.manifold = 2;
    params.m = 4;
    params.alpha = 1e-4;
    params.beta = 0.999;
    params.minstepsize = eps;
    params.maxstepsize = 200;
    params.retraction = Retraction;
    params.vector_transport = VT;
    params.linesearch = 2;
    params.nu = 0;
    times = 0;
    total_timecost = 0;
    fns.phi = @()phi(pp);
    while(total_timecost < 60 && times < 3)
        fprintf('%d: ', times);
        [X, F, G, T, timecost, nf, ng, nR, nH, nV, nVp] = driver_Brockett(B, N, method, fns, params);
        total_timecost = total_timecost + timecost;
        times = times + 1;
    end
    ave_time = total_timecost / times;
    fprintf('average time cost of %d times is %f. \n', times, total_timecost / times);
end

function p = phi(p)
end
